import logging
import telebot

from botik.input.message_handlers.events import events
from botik.input.message_handlers.raw_message_handlers import RawMessageHandlers

from botik_telebot.api.attachment_handlers.photo_attachment_handler import PhotoAttachmentHandler


class RawMessageHandlers(RawMessageHandlers):
    def _initialize_handlers(self, bot):
        bot.message_handler(content_types=['text'])(self.message_reply)
        bot.message_handler(content_types=['photo'])(self.photo_reply)
        bot.message_handler(content_types=['contact'])(self.phone_reply)
        bot.message_handler(content_types=['location'])(self.location_reply)
        bot.message_handler(commands=['start'])(self.start_reply)

        bot.callback_query_handler(func=lambda call: True)(self.callbacks_handle)

    async def _get_user_from_message(self, message):
        user_id = message.from_user.id
        return await self._get_user_from_id(user_id)

    async def callbacks_handle(self, call):
        data = call.data

        user_id = call.from_user.id
        user = await self._get_user_from_id(user_id)

        await self.user_input.forward_inline_button(user, data)

    async def location_reply(self, message):
        user = await self._get_user_from_message(message)
        location = message.location

        await user.storage.set("location", location)
        await events.geo_share(user, location)

    async def photo_reply(self, message):
        user = await self._get_user_from_message(message)
        handler = PhotoAttachmentHandler(self.bot)
        photo = await handler.from_server(message)
        await events.got_attachment(user, photo)

    async def phone_reply(self, message):
        user = await self._get_user_from_message(message)
        number = message.contact.phone_number
        logging.debug(f"Got a number! {number}")

        await user.storage.set("phone", number)
        await events.contact_share(user, number)



from botik.input.keyboard.markup_factory import KeyboardMarkupFactory
from botik_telebot.input.keyboard.keyboard_markup import TgKeyboardMarkup


class TgKeyboardMarkupFactory(KeyboardMarkupFactory):
    def create(self, **native_args):
        return TgKeyboardMarkup(self.button_factory, native_args)



import io

from PIL import Image


class PhotoAttachmentHandler:
    def __init__(self, bot):
        self.bot = bot

    def to_server(self, photo: Image.Image):
        img_bytes = io.BytesIO()
        photo.save(img_bytes, 'JPEG', subsampling=0, quality=95)
        img_bytes.seek(0)
        return img_bytes

    async def from_server(self, message):
        file_id = message.photo[-1].file_id
        file_info = await self.bot.get_file(file_id)
        downloaded_file = await self.bot.download_file(file_info.file_path)
        return Image.open(io.BytesIO(downloaded_file))



from botik.input.keyboard.button.button_factory import ButtonFactory
from botik_telebot.input.keyboard.button.button import TgButton


class TgButtonFactory(ButtonFactory):
    button_type = TgButton



from botik.api.send_message import SendMessage
from botik_telebot.api.attachment_handlers.photo_attachment_handler import PhotoAttachmentHandler
from PIL import Image


class TgSendMessage(SendMessage):

    async def send(self, user, text, attachment=None):
        if attachment and isinstance(attachment, Image.Image):
            handler = PhotoAttachmentHandler(self.bot)
            photo = handler.to_server(attachment)
            await self.bot.send_photo(user.id, photo=photo, caption=text)
        else:
            await self.bot.send_message(user.id, text)

    async def send_with_keyboard(self, user, text, keyboard):
        markup = keyboard.get_native_markup()
        await self.bot.send_message(user.id, text, reply_markup=markup)



from telebot import types

from botik.input.keyboard.button.button import Button
from botik.input.keyboard.button.button_function import ButtonFunction


class TgButton(Button):

    def _create_native(self):
        text = self.get_text()
        req_phone = self.button_function == ButtonFunction.request_phone
        req_location = self.button_function == ButtonFunction.request_location

        if self.inline:
            self.native_data = types.InlineKeyboardButton(text, callback_data=self.get_text(),
                                                          request_contact=req_phone, request_location=req_location,
                                                          **self.native_args)
        else:
            self.native_data = types.KeyboardButton(text, request_contact=req_phone, request_location=req_location,
                                                    **self.native_args)



import asyncio

from botik.api import set_api
from botik.app import App
from botik.navigation import navigator
from botik.page.page_factory import PageFactory
from botik_telebot.api.api import TgApi
from botik_telebot.input.message_handlers.raw_message_handlers import RawMessageHandlers
from botik_telebot.page.page_factory import TgPageFactory


class TgApp(App):
    def start(self):
        self.initialize()
        loop = asyncio.get_event_loop()
        loop.create_task(self.bot.polling())
        loop.run_forever()

    def __init__(self, bot, start_callback=None):
        super().__init__(bot)
        RawMessageHandlers(bot, start_callback,
                           self.users, self.user_input)

    def initialize(self):
        api = TgApi(self.bot)
        set_api(api)

        self._page_fac: PageFactory = TgPageFactory()
        navigator.initialize(self._page_fac, self.pages_data)



from botik.page.page_data import PageData
from botik.page.page_factory import PageFactory
from botik_telebot.input.keyboard.button.button_factory import TgButtonFactory
from botik_telebot.input.keyboard.markup_factory import TgKeyboardMarkupFactory


class TgPageFactory(PageFactory):
    def _make_dependencies(self, page_data: PageData):
        button_factory = TgButtonFactory(page_data.inline)
        self.markup_factory = TgKeyboardMarkupFactory(button_factory)



from telebot import types

from botik.input.keyboard.keyboard_markup import KeyboardMarkup


class TgKeyboardMarkup(KeyboardMarkup):
    def get_native_markup(self):
        return self._make()

    def _make(self):
        if self.inline:
            self._markup = types.InlineKeyboardMarkup(**self.native_args)
        else:
            self._markup = types.ReplyKeyboardMarkup(one_time_keyboard=self.one_time, **self.native_args)

        for i, row in enumerate(self.rows):
            for data in row:
                self._markup.row(data)
        return self._markup



from botik.api.api import Api
from botik_telebot.api.api_type import TgApiType
from botik_telebot.api.send_message import TgSendMessage


class TgApi(Api):
    def __init__(self, bot):
        self.bot = bot
        self.msg = TgSendMessage(bot)
        self.api_type = TgApiType()
